order = [
    'tpDcc.libs.qt.widgets.library.consts',
    'tpDcc.libs.qt.widgets.library.exceptions',
    'tpDcc.libs.qt.widgets.library.manager',
    'tpDcc.libs.qt.widgets.library.utils',
    'tpDcc.libs.qt.widgets.library.library',
    'tpDcc.libs.qt.widgets.library.widgets',
    'tpDcc.libs.qt.widgets.library.mixin',
    'tpDcc.libs.qt.widgets.library.savewidget',
    'tpDcc.libs.qt.widgets.library.loadwidget',
    'tpDcc.libs.qt.widgets.library.items',
    'tpDcc.libs.qt.widgets.library.listview',
    'tpDcc.libs.qt.widgets.library.treewidget',
    'tpDcc.libs.qt.widgets.library.viewer',
    'tpDcc.libs.qt.widgets.library.window'
]
