#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains awesome icons definitions
"""

from __future__ import print_function, division, absolute_import


tags = {
    "500px": u"\uf26e",
    "adjust": u"\uf042",
    "adn": u"\uf170",
    "align-center": u"\uf037",
    "align-justify": u"\uf039",
    "align-left": u"\uf036",
    "align-right": u"\uf038",
    "amazon": u"\uf270",
    "ambulance": u"\uf0f9",
    "american-sign-language-interpreting": u"\uf2a3",
    "anchor": u"\uf13d",
    "android": u"\uf17b",
    "angellist": u"\uf209",
    "angle-double-down": u"\uf103",
    "angle-double-left": u"\uf100",
    "angle-double-right": u"\uf101",
    "angle-double-up": u"\uf102",
    "angle-down": u"\uf107",
    "angle-left": u"\uf104",
    "angle-right": u"\uf105",
    "angle-up": u"\uf106",
    "apple": u"\uf179",
    "archive": u"\uf187",
    "area-chart": u"\uf1fe",
    "arrow-circle-down": u"\uf0ab",
    "arrow-circle-left": u"\uf0a8",
    "arrow-circle-o-down": u"\uf01a",
    "arrow-circle-o-left": u"\uf190",
    "arrow-circle-o-right": u"\uf18e",
    "arrow-circle-o-up": u"\uf01b",
    "arrow-circle-right": u"\uf0a9",
    "arrow-circle-up": u"\uf0aa",
    "arrow-down": u"\uf063",
    "arrow-left": u"\uf060",
    "arrow-right": u"\uf061",
    "arrow-up": u"\uf062",
    "arrows": u"\uf047",
    "arrows-alt": u"\uf0b2",
    "arrows-h": u"\uf07e",
    "arrows-v": u"\uf07d",
    "asl-interpreting (alias)": u"\uf2a3",
    "assistive-listening-systems": u"\uf2a2",
    "asterisk": u"\uf069",
    "at": u"\uf1fa",
    "audio-description": u"\uf29e",
    "automobile (alias)": u"\uf1b9",
    "backward": u"\uf04a",
    "balance-scale": u"\uf24e",
    "ban": u"\uf05e",
    "bank (alias)": u"\uf19c",
    "bar-chart": u"\uf080",
    "bar-chart-o (alias)": u"\uf080",
    "barcode": u"\uf02a",
    "bars": u"\uf0c9",
    "battery-0 (alias)": u"\uf244",
    "battery-1 (alias)": u"\uf243",
    "battery-2 (alias)": u"\uf242",
    "battery-3 (alias)": u"\uf241",
    "battery-4 (alias)": u"\uf240",
    "battery-empty": u"\uf244",
    "battery-full": u"\uf240",
    "battery-half": u"\uf242",
    "battery-quarter": u"\uf243",
    "battery-three-quarters": u"\uf241",
    "bed": u"\uf236",
    "beer": u"\uf0fc",
    "behance": u"\uf1b4",
    "behance-square": u"\uf1b5",
    "bell": u"\uf0f3",
    "bell-o": u"\uf0a2",
    "bell-slash": u"\uf1f6",
    "bell-slash-o": u"\uf1f7",
    "bicycle": u"\uf206",
    "binoculars": u"\uf1e5",
    "birthday-cake": u"\uf1fd",
    "bitbucket": u"\uf171",
    "bitbucket-square": u"\uf172",
    "bitcoin (alias)": u"\uf15a",
    "black-tie": u"\uf27e",
    "blind": u"\uf29d",
    "bluetooth": u"\uf293",
    "bluetooth-b": u"\uf294",
    "bold": u"\uf032",
    "bolt": u"\uf0e7",
    "bomb": u"\uf1e2",
    "book": u"\uf02d",
    "bookmark": u"\uf02e",
    "bookmark-o": u"\uf097",
    "braille": u"\uf2a1",
    "briefcase": u"\uf0b1",
    "btc": u"\uf15a",
    "bug": u"\uf188",
    "building": u"\uf1ad",
    "building-o": u"\uf0f7",
    "bullhorn": u"\uf0a1",
    "bullseye": u"\uf140",
    "bus": u"\uf207",
    "buysellads": u"\uf20d",
    "cab (alias)": u"\uf1ba",
    "calculator": u"\uf1ec",
    "calendar": u"\uf073",
    "calendar-check-o": u"\uf274",
    "calendar-minus-o": u"\uf272",
    "calendar-o": u"\uf133",
    "calendar-plus-o": u"\uf271",
    "calendar-times-o": u"\uf273",
    "camera": u"\uf030",
    "camera-retro": u"\uf083",
    "car": u"\uf1b9",
    "caret-down": u"\uf0d7",
    "caret-left": u"\uf0d9",
    "caret-right": u"\uf0da",
    "caret-square-o-down": u"\uf150",
    "caret-square-o-left": u"\uf191",
    "caret-square-o-right": u"\uf152",
    "caret-square-o-up": u"\uf151",
    "caret-up": u"\uf0d8",
    "cart-arrow-down": u"\uf218",
    "cart-plus": u"\uf217",
    "cc": u"\uf20a",
    "cc-amex": u"\uf1f3",
    "cc-diners-club": u"\uf24c",
    "cc-discover": u"\uf1f2",
    "cc-jcb": u"\uf24b",
    "cc-mastercard": u"\uf1f1",
    "cc-paypal": u"\uf1f4",
    "cc-stripe": u"\uf1f5",
    "cc-visa": u"\uf1f0",
    "certificate": u"\uf0a3",
    "chain (alias)": u"\uf0c1",
    "chain-broken": u"\uf127",
    "check": u"\uf00c",
    "check-circle": u"\uf058",
    "check-circle-o": u"\uf05d",
    "check-square": u"\uf14a",
    "check-square-o": u"\uf046",
    "chevron-circle-down": u"\uf13a",
    "chevron-circle-left": u"\uf137",
    "chevron-circle-right": u"\uf138",
    "chevron-circle-up": u"\uf139",
    "chevron-down": u"\uf078",
    "chevron-left": u"\uf053",
    "chevron-right": u"\uf054",
    "chevron-up": u"\uf077",
    "child": u"\uf1ae",
    "chrome": u"\uf268",
    "circle": u"\uf111",
    "circle-o": u"\uf10c",
    "circle-o-notch": u"\uf1ce",
    "circle-thin": u"\uf1db",
    "clipboard": u"\uf0ea",
    "clock-o": u"\uf017",
    "clone": u"\uf24d",
    "close (alias)": u"\uf00d",
    "cloud": u"\uf0c2",
    "cloud-download": u"\uf0ed",
    "cloud-upload": u"\uf0ee",
    "cny (alias)": u"\uf157",
    "code": u"\uf121",
    "code-fork": u"\uf126",
    "codepen": u"\uf1cb",
    "codiepie": u"\uf284",
    "coffee": u"\uf0f4",
    "cog": u"\uf013",
    "cogs": u"\uf085",
    "columns": u"\uf0db",
    "comment": u"\uf075",
    "comment-o": u"\uf0e5",
    "commenting": u"\uf27a",
    "commenting-o": u"\uf27b",
    "comments": u"\uf086",
    "comments-o": u"\uf0e6",
    "compass": u"\uf14e",
    "compress": u"\uf066",
    "connectdevelop": u"\uf20e",
    "contao": u"\uf26d",
    "copy (alias)": u"\uf0c5",
    "copyright": u"\uf1f9",
    "creative-commons": u"\uf25e",
    "credit-card": u"\uf09d",
    "credit-card-alt": u"\uf283",
    "crop": u"\uf125",
    "crosshairs": u"\uf05b",
    "css3": u"\uf13c",
    "cube": u"\uf1b2",
    "cubes": u"\uf1b3",
    "cut (alias)": u"\uf0c4",
    "cutlery": u"\uf0f5",
    "dashboard (alias)": u"\uf0e4",
    "dashcube": u"\uf210",
    "database": u"\uf1c0",
    "deaf": u"\uf2a4",
    "deafness (alias)": u"\uf2a4",
    "dedent (alias)": u"\uf03b",
    "delicious": u"\uf1a5",
    "desktop": u"\uf108",
    "deviantart": u"\uf1bd",
    "diamond": u"\uf219",
    "digg": u"\uf1a6",
    "dollar (alias)": u"\uf155",
    "dot-circle-o": u"\uf192",
    "download": u"\uf019",
    "dribbble": u"\uf17d",
    "dropbox": u"\uf16b",
    "drupal": u"\uf1a9",
    "edge": u"\uf282",
    "edit (alias)": u"\uf044",
    "eject": u"\uf052",
    "ellipsis-h": u"\uf141",
    "ellipsis-v": u"\uf142",
    "empire": u"\uf1d1",
    "envelope": u"\uf0e0",
    "envelope-o": u"\uf003",
    "envelope-square": u"\uf199",
    "envira": u"\uf299",
    "eraser": u"\uf12d",
    "eur": u"\uf153",
    "euro (alias)": u"\uf153",
    "exchange": u"\uf0ec",
    "exclamation": u"\uf12a",
    "exclamation-circle": u"\uf06a",
    "exclamation-triangle": u"\uf071",
    "expand": u"\uf065",
    "expeditedssl": u"\uf23e",
    "external-link": u"\uf08e",
    "external-link-square": u"\uf14c",
    "eye": u"\uf06e",
    "eye-slash": u"\uf070",
    "eyedropper": u"\uf1fb",
    "fa (alias)": u"\uf2b4",
    "facebook": u"\uf09a",
    "facebook-f (alias)": u"\uf09a",
    "facebook-official": u"\uf230",
    "facebook-square": u"\uf082",
    "fast-backward": u"\uf049",
    "fast-forward": u"\uf050",
    "fax": u"\uf1ac",
    "feed (alias)": u"\uf09e",
    "female": u"\uf182",
    "fighter-jet": u"\uf0fb",
    "file": u"\uf15b",
    "file-archive-o": u"\uf1c6",
    "file-audio-o": u"\uf1c7",
    "file-code-o": u"\uf1c9",
    "file-excel-o": u"\uf1c3",
    "file-image-o": u"\uf1c5",
    "file-movie-o (alias)": u"\uf1c8",
    "file-o": u"\uf016",
    "file-pdf-o": u"\uf1c1",
    "file-photo-o (alias)": u"\uf1c5",
    "file-picture-o (alias)": u"\uf1c5",
    "file-powerpoint-o": u"\uf1c4",
    "file-sound-o (alias)": u"\uf1c7",
    "file-text": u"\uf15c",
    "file-text-o": u"\uf0f6",
    "file-video-o": u"\uf1c8",
    "file-word-o": u"\uf1c2",
    "file-zip-o (alias)": u"\uf1c6",
    "files-o": u"\uf0c5",
    "film": u"\uf008",
    "filter": u"\uf0b0",
    "fire": u"\uf06d",
    "fire-extinguisher": u"\uf134",
    "firefox": u"\uf269",
    "first-order": u"\uf2b0",
    "flag": u"\uf024",
    "flag-checkered": u"\uf11e",
    "flag-o": u"\uf11d",
    "flash (alias)": u"\uf0e7",
    "flask": u"\uf0c3",
    "flickr": u"\uf16e",
    "floppy-o": u"\uf0c7",
    "folder": u"\uf07b",
    "folder-o": u"\uf114",
    "folder-open": u"\uf07c",
    "folder-open-o": u"\uf115",
    "font": u"\uf031",
    "font-awesome": u"\uf2b4",
    "fonticons": u"\uf280",
    "fort-awesome": u"\uf286",
    "forumbee": u"\uf211",
    "forward": u"\uf04e",
    "foursquare": u"\uf180",
    "frown-o": u"\uf119",
    "futbol-o": u"\uf1e3",
    "gamepad": u"\uf11b",
    "gavel": u"\uf0e3",
    "gbp": u"\uf154",
    "ge (alias)": u"\uf1d1",
    "gear (alias)": u"\uf013",
    "gears (alias)": u"\uf085",
    "genderless": u"\uf22d",
    "get-pocket": u"\uf265",
    "gg": u"\uf260",
    "gg-circle": u"\uf261",
    "gift": u"\uf06b",
    "git": u"\uf1d3",
    "git-square": u"\uf1d2",
    "github": u"\uf09b",
    "github-alt": u"\uf113",
    "github-square": u"\uf092",
    "gitlab": u"\uf296",
    "gittip (alias)": u"\uf184",
    "glass": u"\uf000",
    "glide": u"\uf2a5",
    "glide-g": u"\uf2a6",
    "globe": u"\uf0ac",
    "google": u"\uf1a0",
    "google-plus": u"\uf0d5",
    "google-plus-circle (alias)": u"\uf2b3",
    "google-plus-official": u"\uf2b3",
    "google-plus-square": u"\uf0d4",
    "google-wallet": u"\uf1ee",
    "graduation-cap": u"\uf19d",
    "gratipay": u"\uf184",
    "group (alias)": u"\uf0c0",
    "h-square": u"\uf0fd",
    "hacker-news": u"\uf1d4",
    "hand-grab-o (alias)": u"\uf255",
    "hand-lizard-o": u"\uf258",
    "hand-o-down": u"\uf0a7",
    "hand-o-left": u"\uf0a5",
    "hand-o-right": u"\uf0a4",
    "hand-o-up": u"\uf0a6",
    "hand-paper-o": u"\uf256",
    "hand-peace-o": u"\uf25b",
    "hand-pointer-o": u"\uf25a",
    "hand-rock-o": u"\uf255",
    "hand-scissors-o": u"\uf257",
    "hand-spock-o": u"\uf259",
    "hand-stop-o (alias)": u"\uf256",
    "hard-of-hearing (alias)": u"\uf2a4",
    "hashtag": u"\uf292",
    "hdd-o": u"\uf0a0",
    "header": u"\uf1dc",
    "headphones": u"\uf025",
    "heart": u"\uf004",
    "heart-o": u"\uf08a",
    "heartbeat": u"\uf21e",
    "history": u"\uf1da",
    "home": u"\uf015",
    "hospital-o": u"\uf0f8",
    "hotel (alias)": u"\uf236",
    "hourglass": u"\uf254",
    "hourglass-1 (alias)": u"\uf251",
    "hourglass-2 (alias)": u"\uf252",
    "hourglass-3 (alias)": u"\uf253",
    "hourglass-end": u"\uf253",
    "hourglass-half": u"\uf252",
    "hourglass-o": u"\uf250",
    "hourglass-start": u"\uf251",
    "houzz": u"\uf27c",
    "html5": u"\uf13b",
    "i-cursor": u"\uf246",
    "ils": u"\uf20b",
    "image (alias)": u"\uf03e",
    "inbox": u"\uf01c",
    "indent": u"\uf03c",
    "industry": u"\uf275",
    "info": u"\uf129",
    "info-circle": u"\uf05a",
    "inr": u"\uf156",
    "instagram": u"\uf16d",
    "institution (alias)": u"\uf19c",
    "internet-explorer": u"\uf26b",
    "intersex (alias)": u"\uf224",
    "ioxhost": u"\uf208",
    "italic": u"\uf033",
    "joomla": u"\uf1aa",
    "jpy": u"\uf157",
    "jsfiddle": u"\uf1cc",
    "key": u"\uf084",
    "keyboard-o": u"\uf11c",
    "krw": u"\uf159",
    "language": u"\uf1ab",
    "laptop": u"\uf109",
    "lastfm": u"\uf202",
    "lastfm-square": u"\uf203",
    "leaf": u"\uf06c",
    "leanpub": u"\uf212",
    "legal (alias)": u"\uf0e3",
    "lemon-o": u"\uf094",
    "level-down": u"\uf149",
    "level-up": u"\uf148",
    "life-bouy (alias)": u"\uf1cd",
    "life-buoy (alias)": u"\uf1cd",
    "life-ring": u"\uf1cd",
    "life-saver (alias)": u"\uf1cd",
    "lightbulb-o": u"\uf0eb",
    "line-chart": u"\uf201",
    "link": u"\uf0c1",
    "linkedin": u"\uf0e1",
    "linkedin-square": u"\uf08c",
    "linux": u"\uf17c",
    "list": u"\uf03a",
    "list-alt": u"\uf022",
    "list-ol": u"\uf0cb",
    "list-ul": u"\uf0ca",
    "location-arrow": u"\uf124",
    "lock": u"\uf023",
    "long-arrow-down": u"\uf175",
    "long-arrow-left": u"\uf177",
    "long-arrow-right": u"\uf178",
    "long-arrow-up": u"\uf176",
    "low-vision": u"\uf2a8",
    "magic": u"\uf0d0",
    "magnet": u"\uf076",
    "mail-forward (alias)": u"\uf064",
    "mail-reply (alias)": u"\uf112",
    "mail-reply-all (alias)": u"\uf122",
    "male": u"\uf183",
    "map": u"\uf279",
    "map-marker": u"\uf041",
    "map-o": u"\uf278",
    "map-pin": u"\uf276",
    "map-signs": u"\uf277",
    "mars": u"\uf222",
    "mars-double": u"\uf227",
    "mars-stroke": u"\uf229",
    "mars-stroke-h": u"\uf22b",
    "mars-stroke-v": u"\uf22a",
    "maxcdn": u"\uf136",
    "meanpath": u"\uf20c",
    "medium": u"\uf23a",
    "medkit": u"\uf0fa",
    "meh-o": u"\uf11a",
    "mercury": u"\uf223",
    "microphone": u"\uf130",
    "microphone-slash": u"\uf131",
    "minus": u"\uf068",
    "minus-circle": u"\uf056",
    "minus-square": u"\uf146",
    "minus-square-o": u"\uf147",
    "mixcloud": u"\uf289",
    "mobile": u"\uf10b",
    "mobile-phone (alias)": u"\uf10b",
    "modx": u"\uf285",
    "money": u"\uf0d6",
    "moon-o": u"\uf186",
    "mortar-board (alias)": u"\uf19d",
    "motorcycle": u"\uf21c",
    "mouse-pointer": u"\uf245",
    "music": u"\uf001",
    "navicon (alias)": u"\uf0c9",
    "neuter": u"\uf22c",
    "newspaper-o": u"\uf1ea",
    "object-group": u"\uf247",
    "object-ungroup": u"\uf248",
    "odnoklassniki": u"\uf263",
    "odnoklassniki-square": u"\uf264",
    "opencart": u"\uf23d",
    "openid": u"\uf19b",
    "opera": u"\uf26a",
    "optin-monster": u"\uf23c",
    "outdent": u"\uf03b",
    "pagelines": u"\uf18c",
    "paint-brush": u"\uf1fc",
    "paper-plane": u"\uf1d8",
    "paper-plane-o": u"\uf1d9",
    "paperclip": u"\uf0c6",
    "paragraph": u"\uf1dd",
    "paste (alias)": u"\uf0ea",
    "pause": u"\uf04c",
    "pause-circle": u"\uf28b",
    "pause-circle-o": u"\uf28c",
    "paw": u"\uf1b0",
    "paypal": u"\uf1ed",
    "pencil": u"\uf040",
    "pencil-square": u"\uf14b",
    "pencil-square-o": u"\uf044",
    "percent": u"\uf295",
    "phone": u"\uf095",
    "phone-square": u"\uf098",
    "photo (alias)": u"\uf03e",
    "picture-o": u"\uf03e",
    "pie-chart": u"\uf200",
    "pied-piper": u"\uf2ae",
    "pied-piper-alt": u"\uf1a8",
    "pied-piper-pp": u"\uf1a7",
    "pinterest": u"\uf0d2",
    "pinterest-p": u"\uf231",
    "pinterest-square": u"\uf0d3",
    "plane": u"\uf072",
    "play": u"\uf04b",
    "play-circle": u"\uf144",
    "play-circle-o": u"\uf01d",
    "plug": u"\uf1e6",
    "plus": u"\uf067",
    "plus-circle": u"\uf055",
    "plus-square": u"\uf0fe",
    "plus-square-o": u"\uf196",
    "power-off": u"\uf011",
    "print": u"\uf02f",
    "product-hunt": u"\uf288",
    "puzzle-piece": u"\uf12e",
    "qq": u"\uf1d6",
    "qrcode": u"\uf029",
    "question": u"\uf128",
    "question-circle": u"\uf059",
    "question-circle-o": u"\uf29c",
    "quote-left": u"\uf10d",
    "quote-right": u"\uf10e",
    "ra (alias)": u"\uf1d0",
    "random": u"\uf074",
    "rebel": u"\uf1d0",
    "recycle": u"\uf1b8",
    "reddit": u"\uf1a1",
    "reddit-alien": u"\uf281",
    "reddit-square": u"\uf1a2",
    "refresh": u"\uf021",
    "registered": u"\uf25d",
    "remove (alias)": u"\uf00d",
    "renren": u"\uf18b",
    "reorder (alias)": u"\uf0c9",
    "repeat": u"\uf01e",
    "reply": u"\uf112",
    "reply-all": u"\uf122",
    "resistance (alias)": u"\uf1d0",
    "retweet": u"\uf079",
    "rmb (alias)": u"\uf157",
    "road": u"\uf018",
    "rocket": u"\uf135",
    "rotate-left (alias)": u"\uf0e2",
    "rotate-right (alias)": u"\uf01e",
    "rouble (alias)": u"\uf158",
    "rss": u"\uf09e",
    "rss-square": u"\uf143",
    "rub": u"\uf158",
    "ruble (alias)": u"\uf158",
    "rupee (alias)": u"\uf156",
    "safari": u"\uf267",
    "save (alias)": u"\uf0c7",
    "scissors": u"\uf0c4",
    "scribd": u"\uf28a",
    "search": u"\uf002",
    "search-minus": u"\uf010",
    "search-plus": u"\uf00e",
    "sellsy": u"\uf213",
    "send (alias)": u"\uf1d8",
    "send-o (alias)": u"\uf1d9",
    "server": u"\uf233",
    "share": u"\uf064",
    "share-alt": u"\uf1e0",
    "share-alt-square": u"\uf1e1",
    "share-square": u"\uf14d",
    "share-square-o": u"\uf045",
    "shekel (alias)": u"\uf20b",
    "sheqel (alias)": u"\uf20b",
    "shield": u"\uf132",
    "ship": u"\uf21a",
    "shirtsinbulk": u"\uf214",
    "shopping-bag": u"\uf290",
    "shopping-basket": u"\uf291",
    "shopping-cart": u"\uf07a",
    "sign-in": u"\uf090",
    "sign-language": u"\uf2a7",
    "sign-out": u"\uf08b",
    "signal": u"\uf012",
    "signing (alias)": u"\uf2a7",
    "simplybuilt": u"\uf215",
    "sitemap": u"\uf0e8",
    "skyatlas": u"\uf216",
    "skype": u"\uf17e",
    "slack": u"\uf198",
    "sliders": u"\uf1de",
    "slideshare": u"\uf1e7",
    "smile-o": u"\uf118",
    "snapchat": u"\uf2ab",
    "snapchat-ghost": u"\uf2ac",
    "snapchat-square": u"\uf2ad",
    "soccer-ball-o (alias)": u"\uf1e3",
    "sort": u"\uf0dc",
    "sort-alpha-asc": u"\uf15d",
    "sort-alpha-desc": u"\uf15e",
    "sort-amount-asc": u"\uf160",
    "sort-amount-desc": u"\uf161",
    "sort-asc": u"\uf0de",
    "sort-desc": u"\uf0dd",
    "sort-down (alias)": u"\uf0dd",
    "sort-numeric-asc": u"\uf162",
    "sort-numeric-desc": u"\uf163",
    "sort-up (alias)": u"\uf0de",
    "soundcloud": u"\uf1be",
    "space-shuttle": u"\uf197",
    "spinner": u"\uf110",
    "spoon": u"\uf1b1",
    "spotify": u"\uf1bc",
    "square": u"\uf0c8",
    "square-o": u"\uf096",
    "stack-exchange": u"\uf18d",
    "stack-overflow": u"\uf16c",
    "star": u"\uf005",
    "star-half": u"\uf089",
    "star-half-empty (alias)": u"\uf123",
    "star-half-full (alias)": u"\uf123",
    "star-half-o": u"\uf123",
    "star-o": u"\uf006",
    "steam": u"\uf1b6",
    "steam-square": u"\uf1b7",
    "step-backward": u"\uf048",
    "step-forward": u"\uf051",
    "stethoscope": u"\uf0f1",
    "sticky-note": u"\uf249",
    "sticky-note-o": u"\uf24a",
    "stop": u"\uf04d",
    "stop-circle": u"\uf28d",
    "stop-circle-o": u"\uf28e",
    "street-view": u"\uf21d",
    "strikethrough": u"\uf0cc",
    "stumbleupon": u"\uf1a4",
    "stumbleupon-circle": u"\uf1a3",
    "subscript": u"\uf12c",
    "subway": u"\uf239",
    "suitcase": u"\uf0f2",
    "sun-o": u"\uf185",
    "superscript": u"\uf12b",
    "support (alias)": u"\uf1cd",
    "table": u"\uf0ce",
    "tablet": u"\uf10a",
    "tachometer": u"\uf0e4",
    "tag": u"\uf02b",
    "tags": u"\uf02c",
    "tasks": u"\uf0ae",
    "taxi": u"\uf1ba",
    "television": u"\uf26c",
    "tencent-weibo": u"\uf1d5",
    "terminal": u"\uf120",
    "text-height": u"\uf034",
    "text-width": u"\uf035",
    "th": u"\uf00a",
    "th-large": u"\uf009",
    "th-list": u"\uf00b",
    "themeisle": u"\uf2b2",
    "thumb-tack": u"\uf08d",
    "thumbs-down": u"\uf165",
    "thumbs-o-down": u"\uf088",
    "thumbs-o-up": u"\uf087",
    "thumbs-up": u"\uf164",
    "ticket": u"\uf145",
    "times": u"\uf00d",
    "times-circle": u"\uf057",
    "times-circle-o": u"\uf05c",
    "tint": u"\uf043",
    "toggle-down (alias)": u"\uf150",
    "toggle-left (alias)": u"\uf191",
    "toggle-off": u"\uf204",
    "toggle-on": u"\uf205",
    "toggle-right (alias)": u"\uf152",
    "toggle-up (alias)": u"\uf151",
    "trademark": u"\uf25c",
    "train": u"\uf238",
    "transgender": u"\uf224",
    "transgender-alt": u"\uf225",
    "trash": u"\uf1f8",
    "trash-o": u"\uf014",
    "tree": u"\uf1bb",
    "trello": u"\uf181",
    "tripadvisor": u"\uf262",
    "trophy": u"\uf091",
    "truck": u"\uf0d1",
    "try": u"\uf195",
    "tty": u"\uf1e4",
    "tumblr": u"\uf173",
    "tumblr-square": u"\uf174",
    "turkish-lira (alias)": u"\uf195",
    "tv (alias)": u"\uf26c",
    "twitch": u"\uf1e8",
    "twitter": u"\uf099",
    "twitter-square": u"\uf081",
    "umbrella": u"\uf0e9",
    "underline": u"\uf0cd",
    "undo": u"\uf0e2",
    "universal-access": u"\uf29a",
    "university": u"\uf19c",
    "unlink (alias)": u"\uf127",
    "unlock": u"\uf09c",
    "unlock-alt": u"\uf13e",
    "unsorted (alias)": u"\uf0dc",
    "upload": u"\uf093",
    "usb": u"\uf287",
    "usd": u"\uf155",
    "user": u"\uf007",
    "user-md": u"\uf0f0",
    "user-plus": u"\uf234",
    "user-secret": u"\uf21b",
    "user-times": u"\uf235",
    "users": u"\uf0c0",
    "venus": u"\uf221",
    "venus-double": u"\uf226",
    "venus-mars": u"\uf228",
    "viacoin": u"\uf237",
    "viadeo": u"\uf2a9",
    "viadeo-square": u"\uf2aa",
    "video-camera": u"\uf03d",
    "vimeo": u"\uf27d",
    "vimeo-square": u"\uf194",
    "vine": u"\uf1ca",
    "vk": u"\uf189",
    "volume-control-phone": u"\uf2a0",
    "volume-down": u"\uf027",
    "volume-off": u"\uf026",
    "volume-up": u"\uf028",
    "warning (alias)": u"\uf071",
    "wechat (alias)": u"\uf1d7",
    "weibo": u"\uf18a",
    "weixin": u"\uf1d7",
    "whatsapp": u"\uf232",
    "wheelchair": u"\uf193",
    "wheelchair-alt": u"\uf29b",
    "wifi": u"\uf1eb",
    "wikipedia-w": u"\uf266",
    "windows": u"\uf17a",
    "won (alias)": u"\uf159",
    "wordpress": u"\uf19a",
    "wpbeginner": u"\uf297",
    "wpforms": u"\uf298",
    "wrench": u"\uf0ad",
    "xing": u"\uf168",
    "xing-square": u"\uf169",
    "y-combinator": u"\uf23b",
    "y-combinator-square (alias)": u"\uf1d4",
    "yahoo": u"\uf19e",
    "yc (alias)": u"\uf23b",
    "yc-square (alias)": u"\uf1d4",
    "yelp": u"\uf1e9",
    "yen (alias)": u"\uf157",
    "yoast": u"\uf2b1",
    "youtube": u"\uf167",
    "youtube-play": u"\uf16a",
    "youtube-square": u"\uf166",
}
