#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constants for tpQtLib
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

# DPI
DEFAULT_DPI = 96

# SPACINGS
SPACING = 2
DEFAULT_SUB_WIDGET_SPACING = 6

# COLORS
DARK_BG_COLOR = tuple([93, 93, 93])
MEDIUM_DARK_BG_COLOR = tuple([73, 73, 73])

# AXISES
AXISES = {
    'x': [255, 0, 0],
    'y': [0, 255, 0],
    'z': [0, 0, 255]
}
